"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExtensionKind = exports.ExtensionKind = void 0;
const ts = require("typescript");
const path = require("path");
var ExtensionKind;
(function (ExtensionKind) {
    ExtensionKind["MultiFunction"] = "MultiFunction";
    ExtensionKind["MultiType"] = "MultiType";
    ExtensionKind["RangeFunction"] = "RangeFunction";
    ExtensionKind["AdditionOperatorType"] = "AdditionOperatorType";
    ExtensionKind["AdditionOperatorMethodType"] = "AdditionOperatorMethodType";
    ExtensionKind["SubtractionOperatorType"] = "SubtractionOperatorType";
    ExtensionKind["SubtractionOperatorMethodType"] = "SubtractionOperatorMethodType";
    ExtensionKind["MultiplicationOperatorType"] = "MultiplicationOperatorType";
    ExtensionKind["MultiplicationOperatorMethodType"] = "MultiplicationOperatorMethodType";
    ExtensionKind["DivisionOperatorType"] = "DivisionOperatorType";
    ExtensionKind["DivisionOperatorMethodType"] = "DivisionOperatorMethodType";
    ExtensionKind["ModuloOperatorType"] = "ModuloOperatorType";
    ExtensionKind["ModuloOperatorMethodType"] = "ModuloOperatorMethodType";
    ExtensionKind["PowerOperatorType"] = "PowerOperatorType";
    ExtensionKind["PowerOperatorMethodType"] = "PowerOperatorMethodType";
    ExtensionKind["FloorDivisionOperatorType"] = "FloorDivisionOperatorType";
    ExtensionKind["FloorDivisionOperatorMethodType"] = "FloorDivisionOperatorMethodType";
    ExtensionKind["BitwiseAndOperatorType"] = "BitwiseAndOperatorType";
    ExtensionKind["BitwiseAndOperatorMethodType"] = "BitwiseAndOperatorMethodType";
    ExtensionKind["BitwiseOrOperatorType"] = "BitwiseOrOperatorType";
    ExtensionKind["BitwiseOrOperatorMethodType"] = "BitwiseOrOperatorMethodType";
    ExtensionKind["BitwiseExclusiveOrOperatorType"] = "BitwiseExclusiveOrOperatorType";
    ExtensionKind["BitwiseExclusiveOrOperatorMethodType"] = "BitwiseExclusiveOrOperatorMethodType";
    ExtensionKind["BitwiseLeftShiftOperatorType"] = "BitwiseLeftShiftOperatorType";
    ExtensionKind["BitwiseLeftShiftOperatorMethodType"] = "BitwiseLeftShiftOperatorMethodType";
    ExtensionKind["BitwiseRightShiftOperatorType"] = "BitwiseRightShiftOperatorType";
    ExtensionKind["BitwiseRightShiftOperatorMethodType"] = "BitwiseRightShiftOperatorMethodType";
    ExtensionKind["ConcatOperatorType"] = "ConcatOperatorType";
    ExtensionKind["ConcatOperatorMethodType"] = "ConcatOperatorMethodType";
    ExtensionKind["LessThanOperatorType"] = "LessThanOperatorType";
    ExtensionKind["LessThanOperatorMethodType"] = "LessThanOperatorMethodType";
    ExtensionKind["GreaterThanOperatorType"] = "GreaterThanOperatorType";
    ExtensionKind["GreaterThanOperatorMethodType"] = "GreaterThanOperatorMethodType";
    ExtensionKind["NegationOperatorType"] = "NegationOperatorType";
    ExtensionKind["NegationOperatorMethodType"] = "NegationOperatorMethodType";
    ExtensionKind["BitwiseNotOperatorType"] = "BitwiseNotOperatorType";
    ExtensionKind["BitwiseNotOperatorMethodType"] = "BitwiseNotOperatorMethodType";
    ExtensionKind["LengthOperatorType"] = "LengthOperatorType";
    ExtensionKind["LengthOperatorMethodType"] = "LengthOperatorMethodType";
})(ExtensionKind = exports.ExtensionKind || (exports.ExtensionKind = {}));
const functionNameToExtensionKind = {
    $multi: ExtensionKind.MultiFunction,
    $range: ExtensionKind.RangeFunction,
};
const typeNameToExtensionKind = {
    LuaMultiReturn: ExtensionKind.MultiType,
    LuaAddition: ExtensionKind.AdditionOperatorType,
    LuaAdditionMethod: ExtensionKind.AdditionOperatorMethodType,
    LuaSubtraction: ExtensionKind.SubtractionOperatorType,
    LuaSubtractionMethod: ExtensionKind.SubtractionOperatorMethodType,
    LuaMultiplication: ExtensionKind.MultiplicationOperatorType,
    LuaMultiplicationMethod: ExtensionKind.MultiplicationOperatorMethodType,
    LuaDivision: ExtensionKind.DivisionOperatorType,
    LuaDivisionMethod: ExtensionKind.DivisionOperatorMethodType,
    LuaModulo: ExtensionKind.ModuloOperatorType,
    LuaModuloMethod: ExtensionKind.ModuloOperatorMethodType,
    LuaPower: ExtensionKind.PowerOperatorType,
    LuaPowerMethod: ExtensionKind.PowerOperatorMethodType,
    LuaFloorDivision: ExtensionKind.FloorDivisionOperatorType,
    LuaFloorDivisionMethod: ExtensionKind.FloorDivisionOperatorMethodType,
    LuaBitwiseAnd: ExtensionKind.BitwiseAndOperatorType,
    LuaBitwiseAndMethod: ExtensionKind.BitwiseAndOperatorMethodType,
    LuaBitwiseOr: ExtensionKind.BitwiseOrOperatorType,
    LuaBitwiseOrMethod: ExtensionKind.BitwiseOrOperatorMethodType,
    LuaBitwiseExclusiveOr: ExtensionKind.BitwiseExclusiveOrOperatorType,
    LuaBitwiseExclusiveOrMethod: ExtensionKind.BitwiseExclusiveOrOperatorMethodType,
    LuaBitwiseLeftShift: ExtensionKind.BitwiseLeftShiftOperatorType,
    LuaBitwiseLeftShiftMethod: ExtensionKind.BitwiseLeftShiftOperatorMethodType,
    LuaBitwiseRightShift: ExtensionKind.BitwiseRightShiftOperatorType,
    LuaBitwiseRightShiftMethod: ExtensionKind.BitwiseRightShiftOperatorMethodType,
    LuaConcat: ExtensionKind.ConcatOperatorType,
    LuaConcatMethod: ExtensionKind.ConcatOperatorMethodType,
    LuaLessThan: ExtensionKind.LessThanOperatorType,
    LuaLessThanMethod: ExtensionKind.LessThanOperatorMethodType,
    LuaGreaterThan: ExtensionKind.GreaterThanOperatorType,
    LuaGreaterThanMethod: ExtensionKind.GreaterThanOperatorMethodType,
    LuaNegation: ExtensionKind.NegationOperatorType,
    LuaNegationMethod: ExtensionKind.NegationOperatorMethodType,
    LuaBitwiseNot: ExtensionKind.BitwiseNotOperatorType,
    LuaBitwiseNotMethod: ExtensionKind.BitwiseNotOperatorMethodType,
    LuaLength: ExtensionKind.LengthOperatorType,
    LuaLengthMethod: ExtensionKind.LengthOperatorMethodType,
};
function isSourceFileFromLanguageExtensions(sourceFile) {
    const extensionDirectory = path.resolve(__dirname, "../../../language-extensions");
    const sourceFileDirectory = path.dirname(path.normalize(sourceFile.fileName));
    return extensionDirectory === sourceFileDirectory;
}
function getExtensionKind(declaration) {
    var _a;
    const sourceFile = declaration.getSourceFile();
    if (isSourceFileFromLanguageExtensions(sourceFile)) {
        if (ts.isFunctionDeclaration(declaration) && ((_a = declaration.name) === null || _a === void 0 ? void 0 : _a.text)) {
            const extensionKind = functionNameToExtensionKind[declaration.name.text];
            if (extensionKind) {
                return extensionKind;
            }
        }
        if (ts.isTypeAliasDeclaration(declaration)) {
            const extensionKind = typeNameToExtensionKind[declaration.name.text];
            if (extensionKind) {
                return extensionKind;
            }
        }
        throw new Error("Unknown extension kind");
    }
}
exports.getExtensionKind = getExtensionKind;
//# sourceMappingURL=language-extensions.js.map